"""TAGS HANDLER.

Doc: https://documentation.mailgun.com/en/latest/api-tags.html
"""

from __future__ import annotations

from os import path
from typing import Any
from urllib.parse import quote


def handle_tags(
    url: Any,
    domain: str | None,
    _method: str | None,
    **kwargs: Any,
) -> Any:
    """Handle Tags.

    :param url: Incoming URL dictionary
    :type url: dict
    :param domain: Incoming domain
    :type domain: str
    :param _method: Incoming request method (but not used here)
    :type _method: str
    :param kwargs: kwargs
    :return: final url for Tags endpoint
    """
    final_keys = path.join("/", *url["keys"]) if url["keys"] else ""
    base = url["base"] + domain + "/"
    keys_without_tags = url["keys"][1:]
    url = url["base"] + domain + final_keys
    if "tag_name" in kwargs:
        if "stats" in final_keys:
            final_keys = path.join("/", *keys_without_tags) if keys_without_tags else ""
            url = base + "tags" + "/" + quote(kwargs["tag_name"]) + final_keys
        else:
            url = url + "/" + quote(kwargs["tag_name"])

    return url



import json
import os
from pathlib import Path

from mailgun.client import Client


key: str = os.environ["APIKEY"]
domain: str = os.environ["DOMAIN"]

client: Client = Client(auth=("api", key))

# Bounces


def get_bounces() -> None:
    """
    GET /<domain>/bounces
    :return:
    """
    req = client.bounces.get(domain=domain)
    print(req.json())


def post_bounces() -> None:
    """
    POST /<domain>/bounces
    :return:
    """
    data = {"address": "test120@gmail.com", "code": 550, "error": "Test error"}
    req = client.bounces.create(data=data, domain=domain)
    print(req.json())


def get_single_bounce() -> None:
    """
    GET /<domain>/bounces/<address>
    :return:
    """
    req = client.bounces.get(domain=domain, bounce_address="test120@gmail.com")
    print(req.json())


def add_multiple_bounces() -> None:
    """
    POST /<domain>/bounces, Content-Type: application/json
    :return:
    """
    data = """[{
        "address": "test121@i.ua",
        "code": "550",
        "error": "Test error2312"
    },
        {
            "address": "test122@gmail.com",
            "code": "550",
            "error": "Test error"
        }]"""
    json_data = json.loads(data)
    for address in json_data:
        req = client.bounces.create(
            data=address, domain=domain, headers={"Content-type": "application/json"}
        )
        print(req.json())


def import_bounce_list() -> None:
    """
    POST /<domain>/bounces/import, Content-Type: multipart/form-data
    :return:
    """
    # It is strongly recommended that you open files in binary mode.
    # Because the Content-Length header may be provided for you,
    # and if it does this value will be set to the number of bytes in the file.
    # Errors may occur if you open the file in text mode.
    files = {
        "bounce_csv": Path(
            "mailgun/doc_tests/files/mailgun_bounces_test.csv"
        ).read_bytes()
    }
    req = client.bounces_import.create(domain=domain, files=files)
    print(req.json())


def delete_single_bounce() -> None:
    """
    DELETE /<domain>/bounces/<address>
    :return:
    """
    req = client.bounces.delete(domain=domain, bounce_address="test122@gmail.com")
    print(req.json())


def delete_bounce_list() -> None:
    """
    DELETE /<domain>/bounces
    :return:
    """
    req = client.bounces.delete(domain=domain)
    print(req.json())


# Unsubscribes


def get_unsubs() -> None:
    """
    GET /<domain>/unsubscribes
    :return:
    """
    req = client.unsubscribes.get(domain=domain)
    print(req.json())


def get_single_unsub() -> None:
    """
    GET /<domain>/unsubscribes/<address>
    :return:
    """
    req = client.unsubscribes.get(domain=domain, unsubscribe_address="test1@gmail.com")
    print(req.json())


def create_single_unsub() -> None:
    """
    POST /<domain>/unsubscribes
    :return:
    """
    data = {"address": "bob@example.com", "tag": "*"}
    req = client.unsubscribes.create(data=data, domain=domain)
    print(req.json())


def create_multiple_unsub() -> None:
    """
    POST /<domain>/unsubscribes, Content-Type: application/json
    :return:
    """
    data = """[{
            "address": "alice@example.com",
            "tags": ["some tag"],
            "created_at": "Thu, 13 Oct 2011 18:02:00 UTC"
        },
        {
            "address": "bob@example.com",
            "tags": ["*"]
        },
        {"address": "carol@example.com"}]"""

    json_data = json.loads(data)
    for address in json_data:
        req = client.unsubscribes.create(
            data=address, domain=domain, headers={"Content-type": "application/json"}
        )
        print(req.json())


def import_list_unsubs() -> None:
    """
    POST /<domain>/unsubscribes/import, Content-Type: multipart/form-data
    :return:
    """
    # It is strongly recommended that you open files in binary mode.
    # Because the Content-Length header may be provided for you,
    # and if it does this value will be set to the number of bytes in the file.
    # Errors may occur if you open the file in text mode.
    files = {
        "unsubscribe_csv": Path(
            "mailgun/doc_tests/files/mailgun_unsubscribes.csv"
        ).read_bytes()
    }
    req = client.unsubscribes_import.create(domain=domain, files=files)
    print(req.json())


def delete_single_unsub() -> None:
    """
    DELETE /<domain>/unsubscribes/<address>
    :return:
    """
    req = client.unsubscribes.delete(
        domain=domain, unsubscribe_address="alice@example.com"
    )
    print(req.json())


def delete_all_unsubs() -> None:
    """
    DELETE /<domain>/unsubscribes/
    :return:
    """
    req = client.unsubscribes.delete(domain=domain)
    print(req.json())


# Complaints


def get_complaints() -> None:
    """
    GET /<domain>/complaints

    :return:
    """
    req = client.complaints.get(domain=domain)
    print(req.json())


def add_complaints() -> None:
    """
    POST /<domain>/complaints
    :return:
    """
    data = {"address": "bob@gmail.com", "tag": "compl_test_tag"}
    req = client.complaints.create(data=data, domain=domain)
    print(req.json())


def get_single_complaint() -> None:
    """
    GET /<domain>/complaints/<address>

    :return:
    """
    req = client.complaints.get(domain=domain, complaint_address="bob@gmail.com")
    print(req.json())


def add_multiple_complaints() -> None:
    """
    POST /<domain>/complaints, Content-Type: application/json
    :return:
    """
    data = """[{
            "address": "alice1@example.com",
            "tags": ["some tag"],
            "created_at": "Thu, 13 Oct 2011 18:02:00 UTC"
        },
        {"address": "carol1@example.com"}]"""
    json_data = json.loads(data)
    for address in json_data:
        req = client.complaints.create(
            data=address, domain=domain, headers={"Content-type": "application/json"}
        )
        print(req.json())


def import_complaint_list() -> None:
    """
    POST /<domain>/complaints/import, Content-Type: multipart/form-data
    :return:
    """
    # It is strongly recommended that you open files in binary mode.
    # Because the Content-Length header may be provided for you,
    # and if it does this value will be set to the number of bytes in the file.
    # Errors may occur if you open the file in text mode.
    files = {
        "complaints_csv": Path(
            "mailgun/doc_tests/files/mailgun_complaints.csv"
        ).read_bytes()
    }
    req = client.complaints_import.create(domain=domain, files=files)
    print(req.json())


def delete_single_complaint() -> None:
    """
    DELETE /<domain>/complaints/<address>
    :return:
    """
    req = client.complaints.delete(
        domain=domain, complaint_address="carol1@example.com"
    )
    print(req.json())


def delete_all_complaints() -> None:
    """
    DELETE /<domain>/complaints/
    :return:
    """
    req = client.complaints.delete(domain=domain)
    print(req.json())


# Whitelists


def get_whitelists() -> None:
    """
    GET /<domain>/whitelists
    :return:
    """
    req = client.whitelists.get(domain=domain)
    print(req.json())


def create_whitelist() -> None:
    """
    POST /<domain>/whitelists
    :return:
    """
    data = {"address": "test122@gmail.com", "tag": "whitel_test"}
    req = client.whitelists.create(data=data, domain=domain)
    print(req.json())


def get_single_whitelist() -> None:
    """
    GET /<domain>/whitelists/<address or domain>
    :return:
    """
    # You can set domain name or address for whitelist_address option
    req = client.whitelists.get(domain=domain, whitelist_address="bob@gmail.com")
    print(req.json())


def import_list_whitelists() -> None:
    """
    POST /<domain>/whitelists/import, Content-Type: multipart/form-data
    :return:
    """
    # It is strongly recommended that you open files in binary mode.
    # Because the Content-Length header may be provided for you,
    # and if it does this value will be set to the number of bytes in the file.
    # Errors may occur if you open the file in text mode.
    files = {
        "whitelist_csv": Path(
            "mailgun/doc_tests/files/mailgun_whitelists.csv"
        ).read_bytes()
    }
    req = client.whitelists_import.create(domain=domain, files=files)
    print(req.json())


def delete_single_whitelist() -> None:
    """
    DELETE /<domain>/whitelists/<address or domain>
    :return:
    """
    req = client.whitelists.delete(domain=domain, whitelist_address="bob@gmail.com")
    print(req.json())


def delete_all_whitelists() -> None:
    """
    DELETE /<domain>/whitelists
    :return:
    """
    req = client.whitelists.delete(domain=domain)
    print(req.json())


if __name__ == "__main__":
    delete_single_whitelist()



"""The `tests` package contains unit and integration tests for the Mailgun API client.

This package ensures that all core functionalities of the Mailgun client, including
authentication, API requests, error handling, and response parsing, work as expected.
Each module within this package tests a specific aspect or component of the client.
"""



"""TEMPLATES HANDLER.

Doc: https://documentation.mailgun.com/en/latest/api-templates.html
"""

from __future__ import annotations

from os import path
from typing import Any

from .error_handler import ApiError


def handle_templates(
    url: dict[str, Any],
    domain: str | None,
    _method: str | None,
    **kwargs: Any,
) -> Any:
    """Handle Templates.

    :param url: Incoming URL dictionary
    :type url: dict
    :param domain: Incoming domain
    :type domain: str
    :param _method: Incoming request method (but not used here)
    :type _method: str
    :param kwargs: kwargs
    :return: final url for Templates endpoint
    :raises: ApiError
    """
    final_keys = path.join("/", *url["keys"]) if url["keys"] else ""
    if "template_name" in kwargs:
        if "versions" in kwargs:
            if kwargs["versions"]:
                if "tag" in kwargs:
                    url = (
                        url["base"]
                        + domain
                        + final_keys
                        + "/"
                        + kwargs["template_name"]
                        + "/versions/"
                        + kwargs["tag"]
                    )
                else:
                    url = (
                        url["base"]
                        + domain
                        + final_keys
                        + "/"
                        + kwargs["template_name"]
                        + "/versions"
                    )
            else:
                raise ApiError("Versions should be True or absent")
        else:
            url = url["base"] + domain + final_keys + "/" + kwargs["template_name"]
    else:
        url = url["base"] + domain + final_keys

    return url



import os

from mailgun.client import Client


key: str = os.environ["APIKEY"]
domain: str = os.environ["DOMAIN"]

client: Client = Client(auth=("api", key))


def get_tags() -> None:
    """
    GET /<domain>/tags
    :return:
    """
    req = client.tags.get(domain=domain)
    print(req.json())


def get_single_tag() -> None:
    """
    GET /<domain>/tags/<tag>
    :return:
    """
    req = client.tags.get(domain=domain, tag_name="Python test")
    print(req.json())


def put_single_tag() -> None:
    """
    PUT /<domain>/tags/<tag>
    :return:
    """
    data = {"description": "Python testtt"}

    req = client.tags.put(domain=domain, tag_name="Python test", data=data)
    print(req.json())


def get_tag_stats() -> None:
    """
    GET /<domain>/tags/<tag>/stats
    :return:
    """
    params = {"event": "accepted"}
    req = client.tags_stats.get(domain=domain, filters=params, tag_name="Python test")
    print(req.json())


def delete_tag() -> None:
    """
    DELETE /<domain>/tags/<tag>
    :return:
    """
    req = client.tags.delete(domain=domain, tag_name="Python test")
    print(req.json())


def get_aggregate_countries() -> None:
    """
    GET /<domain>/tags/<tag>/stats/aggregates/countries
    :return:
    """
    req = client.tags_stats_aggregates_countries.get(
        domain=domain, tag_name="September newsletter"
    )
    print(req.json())


def get_aggregate_providers() -> None:
    """
    GET /<domain>/tags/<tag>/stats/aggregates/providers
    :return:
    """
    req = client.tags_stats_aggregates_providers.get(
        domain=domain, tag_name="September newsletter"
    )
    print(req.json())


def get_aggregate_devices() -> None:
    """
    GET /<domain>/tags/<tag>/stats/aggregates/devices
    :return:
    """
    req = client.tags_stats_aggregates_devices.get(
        domain=domain, tag_name="September newsletter"
    )
    print(req.json())


if __name__ == "__main__":
    get_aggregate_devices()



from __future__ import annotations

import os

from mailgun.client import Client


key: str = os.environ["APIKEY"]
domain: str = os.environ["DOMAIN"]

client: Client = Client(auth=("api", key))


def get_webhooks() -> None:
    """
    GET /domains/<domain>/webhooks
    :return:
    """
    req = client.domains_webhooks.get(domain=domain)
    print(req.json())


def create_webhook() -> None:
    """
    POST /domains/<domain>/webhooks
    :return:
    """
    data = {"id": "clicked", "url": ["https://facebook.com"]}
    #
    req = client.domains_webhooks.create(domain=domain, data=data)
    print(req.json())


def get_webhook() -> None:
    """
    GET /domains/<domain>/webhooks/<webhookname>
    :return:
    """
    req = client.domains_webhooks_clicked.get(domain=domain)
    print(req.json())


def put_webhook() -> None:
    """
    PUT /domains/<domain>/webhooks/<webhookname>
    :return:
    """
    data = {"id": "clicked", "url": ["https://facebook.com", "https://google.com"]}

    req = client.domains_webhooks_clicked.put(domain=domain, data=data)
    print(req.json())


def delete_webhook() -> None:
    """
    DELETE /domains/<domain>/webhooks/<webhookname>
    :return:
    """
    req = client.domains_webhooks_clicked.delete(domain=domain)
    print(req.json())


if __name__ == "__main__":
    get_webhooks()



"""SUPPRESSION HANDLER.

Doc: https://documentation.mailgun.com/en/latest/api-suppressions.html
"""

from __future__ import annotations

from os import path
from typing import Any


def handle_bounces(
    url: dict[str, Any],
    domain: str | None,
    _method: str | None,
    **kwargs: Any,
) -> Any:
    """Handle Bounces.

    :param url: Incoming URL dictionary
    :type url: dict
    :param domain: Incoming domain
    :type domain: str
    :param _method: Incoming request method (it's not being used for this handler)
    :type _method: str
    :param kwargs: kwargs
    :return: final url for Bounces endpoint
    """
    final_keys = path.join("/", *url["keys"]) if url["keys"] else ""
    if "bounce_address" in kwargs:
        url = url["base"] + domain + final_keys + "/" + kwargs["bounce_address"]
    else:
        url = url["base"] + domain + final_keys
    return url


def handle_unsubscribes(
    url: dict[str, Any],
    domain: str | None,
    _method: str | None,
    **kwargs: Any,
) -> Any:
    """Handle Unsubscribes.

    :param url: Incoming URL dictionary
    :type url: dict
    :param domain: Incoming domain
    :type domain: str
    :param _method: Incoming request method (it's not being used for this handler)
    :type _method: str
    :param kwargs: kwargs
    :return: final url for Unsubscribes endpoint
    """
    final_keys = path.join("/", *url["keys"]) if url["keys"] else ""
    if "unsubscribe_address" in kwargs:
        url = url["base"] + domain + final_keys + "/" + kwargs["unsubscribe_address"]
    else:
        url = url["base"] + domain + final_keys
    return url


def handle_complaints(
    url: dict[str, Any],
    domain: str | None,
    _method: str | None,
    **kwargs: Any,
) -> Any:
    """Handle Complaints.

    :param url: Incoming URL dictionary
    :type url: dict
    :param domain: Incoming domain
    :type domain: str
    :param _method: Incoming request method (it's not being used for this handler)
    :type _method: str
    :param kwargs: kwargs
    :return: final url for Complaints endpoint
    """
    final_keys = path.join("/", *url["keys"]) if url["keys"] else ""
    if "complaint_address" in kwargs:
        url = url["base"] + domain + final_keys + "/" + kwargs["complaint_address"]
    else:
        url = url["base"] + domain + final_keys
    return url


def handle_whitelists(
    url: dict[str, Any],
    domain: str | None,
    _method: str | None,
    **kwargs: Any,
) -> Any:
    """Handle Whitelists.

    :param url: Incoming URL dictionary
    :type url: dict
    :param domain: Incoming domain
    :type domain: str
    :param _method: Incoming request method (it's not being used for this handler)
    :type _method: str
    :param kwargs: kwargs
    :return: final url for Whitelists endpoint
    """
    final_keys = path.join("/", *url["keys"]) if url["keys"] else ""
    if "whitelist_address" in kwargs:
        url = url["base"] + domain + final_keys + "/" + kwargs["whitelist_address"]
    else:
        url = url["base"] + domain + final_keys

    return url



"""ERROR HANDLER.

Exceptions:
    - ApiError: Base exception for API errors.
"""


class ApiError(Exception):
    """Base class for all API-related errors.

    This exception serves as the root for all custom API error types,
    allowing for more specific error handling based on the type of API
    failure encountered.
    """
